<?php

namespace Drupal\available_translation\Plugin\Block;

use Drupal\available_translation\Service\AvailableTranslationService;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides an Available Translation block.
 *
 * @Block(
 *   id = "available_translation",
 *   admin_label = @Translation("Available Translation"),
 *   category = @Translation("Menus")
 * )
 */
class AvailableTranslationBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Service for building the available translation links.
   *
   * @var \Drupal\available_translation\Service\AvailableTranslationService
   */
  protected $service;

  /**
   * AvailableTranslationBlock constructor.
   *
   * @param array $configuration
   *   The plugin configuration.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\available_translation\Service\AvailableTranslationService $service
   *   Service for building the available translation links.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, AvailableTranslationService $service) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->service = $service;
  }

  /**
   * Creates an instance of the plugin.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container to pull out services used in the plugin.
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   *
   * @return static
   *   Returns an instance of this plugin.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('available_translation.service')
    );
  }

  /**
   * Gets default configuration for this plugin.
   *
   * @return array
   *   An associative array with the default configuration.
   */
  public function defaultConfiguration() {
    return [
      'delimiter' => ' | ',
      'position' => 'left',
    ];
  }

  /**
   * Returns the configuration form elements specific to this block plugin.
   *
   * @param array $form
   *   The form definition array for the block configuration form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The renderable form array representing the entire configuration form.
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $config = $this->getConfiguration();
    $default_config = $this->defaultConfiguration();

    $form['delimiter'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Delimiter'),
      '#description' => $this->t('Example: " | " for "Armenian | English | French"'),
      '#default_value' => $config['delimiter'] ?? $default_config['delimiter'],
    ];

    $form['position'] = [
      '#type' => 'radios',
      '#title' => $this->t('Position'),
      '#options' => [
        'left' => $this->t("Left"),
        'right' => $this->t("Right"),
      ],
      '#default_value' => $config['position'] ?? $default_config['position'],
    ];

    return $form;
  }

  /**
   * Adds submission handling for the block form.
   *
   * @param array $form
   *   The form definition array for the full block configuration form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $values = $form_state->getValues();
    $this->setConfiguration([
      'delimiter' => $values['delimiter'],
      'position' => $values['position'],
    ]);
  }

  /**
   * Builds and returns the renderable array for this block plugin.
   *
   * @return array
   *   A renderable array representing the content of the block.
   */
  public function build() {
    $config = $this->getConfiguration();
    $output = $this->service->getContent($config['delimiter']);
    $output = $output ? $this->t("Available in:") . " " . $output : $this->t("No translation available!");
    return [
      '#theme' => 'available_translation_template',
      '#text' => $output,
      '#position' => $config['position'],
      '#attached' => [
        'library' => ['available_translation/available_translation'],
      ],
      '#cache' => [
        'max-age' => 0,
      ],
    ];
  }

  /**
   * Indicates whether the block should be shown.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The user session for which to check access.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   The access result.
   */
  protected function blockAccess(AccountInterface $account) {
    if (!$account->hasPermission("access available_translation links")) {
      return AccessResult::forbidden();
    }
    return parent::blockAccess($account);
  }

}
